/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
/// <reference types="node" />
/// <reference types="jest__types" />
import type { Context } from 'vm';
import { JSDOM } from 'jsdom';
import type { EnvironmentContext, JestEnvironment, JestEnvironmentConfig } from '@jest/environment';
import { LegacyFakeTimers, ModernFakeTimers } from '@jest/fake-timers';
import type { Global } from '@jest/types';
import { ModuleMocker } from 'jest-mock';
declare type Win = Window & Global.Global & {
    Error: {
        stackTraceLimit: number;
    };
};
export default class JSDOMEnvironment implements JestEnvironment<number> {
    dom: JSDOM | null;
    fakeTimers: LegacyFakeTimers<number> | null;
    fakeTimersModern: ModernFakeTimers | null;
    global: Win;
    private errorEventListener;
    moduleMocker: ModuleMocker | null;
    customExportConditions: string[];
    constructor(config: JestEnvironmentConfig, context: EnvironmentContext);
    setup(): Promise<void>;
    teardown(): Promise<void>;
    exportConditions(): Array<string>;
    getVmContext(): Context | null;
}
export declare const TestEnvironment: typeof JSDOMEnvironment;
export {};
