/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
/// <reference types="node" />
/// <reference types="jest__types" />
import { Context } from 'vm';
import type { EnvironmentContext, JestEnvironment, JestEnvironmentConfig } from '@jest/environment';
import { LegacyFakeTimers, ModernFakeTimers } from '@jest/fake-timers';
import type { Global } from '@jest/types';
import { ModuleMocker } from 'jest-mock';
declare type Timer = {
    id: number;
    ref: () => Timer;
    unref: () => Timer;
};
export default class NodeEnvironment implements JestEnvironment<Timer> {
    context: Context | null;
    fakeTimers: LegacyFakeTimers<Timer> | null;
    fakeTimersModern: ModernFakeTimers | null;
    global: Global.Global;
    moduleMocker: ModuleMocker | null;
    customExportConditions: string[];
    constructor(config: JestEnvironmentConfig, _context: EnvironmentContext);
    setup(): Promise<void>;
    teardown(): Promise<void>;
    exportConditions(): Array<string>;
    getVmContext(): Context | null;
}
export declare const TestEnvironment: typeof NodeEnvironment;
export {};
